'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

/**
 * Created by xy on 15/4/13.
 */

var React = require('react');
var Const = require('uxcore-const');
var classnames = require('classnames');
var deepcopy = require('lodash/cloneDeep');
var deepEqual = require('deep-equal');
var DateField = require('uxcore-date-cell-field');
var CheckField = require('uxcore-checkbox-cell-field');

var CollapsedButton = require('../CollapsedButton');
var CheckBox = require('./CheckBox');
var Radio = require('./Radio');
var TextField = require('../CellField/TextField');
var SelectField = require('../CellField/SelectField');
var RadioField = require('../CellField/RadioField');
var util = require('../util');

var fieldsMap = {
  select: SelectField,
  text: TextField,
  radio: RadioField,
  date: DateField,
  check: CheckField
};

var Cell = function (_React$Component) {
  _inherits(Cell, _React$Component);

  function Cell() {
    _classCallCheck(this, Cell);

    return _possibleConstructorReturn(this, _React$Component.apply(this, arguments));
  }

  // componentDidMount() {
  //   const me = this;
  //   if (me.props.column.type === 'checkbox'
  //     || me.props.column.type === 'checkboxSelector'
  //     || me.props.column.type === 'radioSelector') {
  //     me.props.changeSelected(me.getCellData(), me.props.rowIndex, true);
  //   }
  // }

  Cell.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
    // 需要考虑的 prop 包括
    // column, rowData, rowIndex(s), index(s), cellIndex(s), hasSubComp(s)
    // rowSelection, actions, mode(s)
    var me = this;
    var shouldUpdate = false;

    // only tree mode has children
    if ('children' in nextProps) {
      return true;
    }

    ['rowIndex', 'index', 'cellIndex', 'hasSubComp', 'mode'].forEach(function (item) {
      if (me.props[item] !== nextProps[item]) {
        shouldUpdate = true;
      }
    });

    if (!shouldUpdate) {
      ['column', 'rowSelection', 'rowData', 'actions'].forEach(function (item) {
        if (!deepEqual(me.props[item], nextProps[item])) {
          shouldUpdate = true;
        }
      });
    }

    return shouldUpdate;
  };

  /**
   * @param actions {Array|Object|Function}
   */


  Cell.prototype.getActionItems = function getActionItems(actions) {
    if ((typeof actions === 'undefined' ? 'undefined' : _typeof(actions)) !== 'object' && typeof actions !== 'function') {
      console.error('Table: Actions should be an object/array/function');
      return [];
    }
    var me = this;
    me.items = [];
    if (typeof actions === 'function') {
      me.items = actions(me.props.rowData);
    } else if (actions instanceof Array) {
      me.items = actions;
    } else {
      Object.keys(actions).forEach(function (item) {
        me.items.push({
          title: item,
          callback: actions[item]
        });
      });
    }
    return me.items;
  };

  Cell.prototype.getEditData = function getEditData() {
    var me = this;
    var column = me.props.column;
    var editKey = column.editKey || column.dataKey;
    return me.props.rowData[editKey];
  };

  Cell.prototype.getCellData = function getCellData(nextProps) {
    var props = nextProps || this.props;
    var column = props.column;
    var cellData = props.rowData[column.dataKey];
    return cellData;
  };

  Cell.prototype.handleCheckChange = function handleCheckChange(e) {
    var me = this;
    me.props.changeSelected(e.target.checked, me.props.rowIndex, false);
  };

  Cell.prototype.handleActionClick = function handleActionClick(cb, e) {
    e.stopPropagation();
    e.preventDefault();
    if (cb) {
      cb();
    }
  };

  Cell.prototype.showSubComp = function showSubComp() {
    this.props.showSubCompCallback.apply();
  };

  /**
   * @param {Object} column current column config
   * @param {Object} rowData current row data
   * @param {String} mode current row mode: edit or view, same as rowData['__mode__']
   */

  Cell.prototype.renderActionItems = function renderActionItems(column, rowData, mode) {
    var me = this;
    var actions = me.getActionItems(column.actions).filter(function (item) {
      return !('mode' in item) || item.mode === mode;
    });
    var items = [];
    actions.forEach(function (action, index) {
      var content = action.render ? action.render(action.title, deepcopy(me.props.rowData)) : action.title;
      if (content) {
        items.push(React.createElement(
          CollapsedButton.Item,
          {
            key: index,
            disabled: typeof action.isDisable === 'function' ? action.isDisable() : false,
            onClick: me.handleActionClick.bind(me, (action.callback || function () {}).bind(me, rowData, me.props.root))
          },
          content
        ));
      }
    });
    return React.createElement(
      CollapsedButton,
      {
        prefixCls: me.props.jsxprefixCls,
        maxLength: column.collapseNum,
        type: column.actionType,
        locale: me.props.locale
      },
      items
    );
  };

  Cell.prototype.renderTreeIcon = function renderTreeIcon() {
    if (this.props.cellIndex === 0 && this.props.hasSubComp) {
      var open = this.props.rowData.showSubComp;
      return React.createElement(
        'span',
        {
          className: 'kuma-uxtable-tree-icon',
          onClick: this.showSubComp.bind(this)
        },
        React.createElement('i', {
          className: classnames({
            'kuma-icon': true,
            'kuma-icon-triangle-right': true,
            'kuma-icon-triangle-right__open': open
          })
        })
      );
    }
    return null;
  };

  Cell.prototype.render = function render() {
    var _classnames;

    var me = this;
    var props = me.props;
    var column = props.column;
    var width = column.width;
    var mode = props.rowData.__mode__;
    var style = {
      width: width || 100,
      textAlign: props.column.align ? props.column.align : 'left'
    };
    var content = deepcopy(props.rowData);
    var renderProps = void 0;

    if (column.type === 'action') {
      content = React.createElement(
        'div',
        { className: 'action-container' },
        me.renderActionItems(column, content, mode)
      );
    } else if (column.type === 'checkbox' || column.type === 'checkboxSelector') {
      style.paddingRight = 4;
      style.paddingLeft = 12;

      var checked = me.getCellData();
      var disable = props.rowSelection.isDisabled ? props.rowSelection.isDisabled(props.rowData) : false;
      if ('disable' in column) {
        disable = column.disable;
      } else if ('isDisable' in column) {
        disable = !!column.isDisable(props.rowData);
      }
      content = React.createElement(CheckBox, {
        disable: disable,
        mode: props.mode,
        align: props.column.align,
        checked: checked,
        onChange: me.handleCheckChange.bind(me)
      });
    } else if (column.type === 'radioSelector') {
      style.paddingRight = 4;
      style.paddingLeft = 12;

      var _checked = me.getCellData();
      var _disable = props.rowSelection.isDisabled ? props.rowSelection.isDisabled(props.rowData) : false;
      if ('disable' in column) {
        _disable = column.disable;
      } else if ('isDisable' in column) {
        _disable = !!column.isDisable(props.rowData);
      }
      content = React.createElement(Radio, {
        disable: _disable,
        mode: props.mode,
        align: props.column.align,
        checked: _checked,
        onChange: me.handleCheckChange.bind(me)
      });
    } else if (column.type === 'treeIcon') {
      content = me.renderTreeIcon();
      style.borderRight = 'none';
    } else if ((column.type === 'custom' || column.type in fieldsMap) && mode === Const.MODE.EDIT && (!('canEdit' in column) || column.canEdit(props.rowData))) {
      // inline edit mode
      renderProps = {
        value: me.getEditData(),
        rowData: props.rowData,
        index: props.index,
        column: column,
        handleDataChange: props.handleDataChange,
        attachCellField: props.attachCellField,
        detachCellField: props.detachCellField
      };
      var Field = void 0;

      if (column.type === 'custom') {
        Field = props.column.customField;
      } else {
        Field = fieldsMap[column.type];
      }
      content = React.createElement(Field, renderProps);
    } else if (column.type === 'money' || column.type === 'card' || column.type === 'cnmobile') {
      content = React.createElement(
        'div',
        {
          className: 'default-cell',
          title: me.getCellData()
        },
        util.formatValue(me.getCellData(), column.type, column.delimiter)
      );
    } else if (column.render) {
      content = column.render.apply(null, [me.getCellData(), content]);
    } else {
      content = React.createElement(
        'div',
        { className: 'default-cell', title: me.getCellData() },
        me.getCellData()
      );
    }

    var child = me.props.children;
    return React.createElement(
      'div',
      {
        className: classnames((_classnames = {}, _defineProperty(_classnames, props.jsxprefixCls, true), _defineProperty(_classnames, 'last', props.last), _classnames)),
        style: style
      },
      child,
      content
    );
  };

  return Cell;
}(React.Component);

Cell.propTypes = {
  cellIndex: React.PropTypes.number,
  hasSubComp: React.PropTypes.bool,
  rowData: React.PropTypes.object,
  jsxprefixCls: React.PropTypes.string,
  showSubCompCallback: React.PropTypes.func
};

Cell.defaultProps = {
  jsxprefixCls: 'kuma-uxtable-cell',
  rowSelection: {}
};

exports["default"] = Cell;
module.exports = exports['default'];