'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

/**
 * Created by xy on 15/4/13.
 */
var Cell = require('./Cell');
var classnames = require('classnames');
var assign = require('object-assign');
var deepEqual = require('deep-equal');
var deepcopy = require('lodash/cloneDeep');
var CheckBox = require('./Cell/CheckBox');
var util = require('./util');

var React = require('react');

var Row = function (_React$Component) {
  _inherits(Row, _React$Component);

  function Row() {
    _classCallCheck(this, Row);

    return _possibleConstructorReturn(this, _React$Component.apply(this, arguments));
  }

  Row.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
    // 需要考虑的 prop 包括
    // columns, rowIndex(s => simple), rowData, index(s),
    // addRowClassName(f => function), rowSelection, subComp(no support), renderSubComp(f), actions
    // mode(s), renderModel(s), fixedColumn(s), levels(s),
    // visible(s), expandedKeys, checkboxColumnKey(s)
    var me = this;
    var shouldUpdate = false;

    ['rowIndex', 'index', 'mode', 'renderModel', 'fixedColumn', 'levels', 'addRowClassName', 'renderSubComp', 'visible', 'checkboxColumnKey', 'locale', 'isHover'].forEach(function (item) {
      if (me.props[item] !== nextProps[item]) {
        shouldUpdate = true;
      }
    });
    if (!shouldUpdate) {
      ['columns', 'rowData', 'rowSelection', 'actions', 'expandedKeys'].forEach(function (item) {
        if (!deepEqual(me.props[item], nextProps[item])) {
          shouldUpdate = true;
        }
      });
    }
    return shouldUpdate;
  };

  Row.prototype.handleClick = function handleClick() {
    this.props.onClick();
  };

  Row.prototype.handleMouseEnter = function handleMouseEnter() {
    this.props.root.handleRowHover(this.props.index, true);
  };

  Row.prototype.handleMouseLeave = function handleMouseLeave() {
    this.props.root.handleRowHover(this.props.index, false);
  };

  Row.prototype.handleDoubleClick = function handleDoubleClick(rowData) {
    var table = this.props.root;
    if (table.props.doubleClickToEdit) {
      table.editRow(deepcopy(rowData));
    }
  };

  Row.prototype.handleTreeCheckChange = function handleTreeCheckChange(e) {
    var me = this;
    me.props.root.changeTreeSelected(e.target.checked, me.props.dataIndex || me.props.index);
  };

  Row.prototype.showSubCompFunc = function showSubCompFunc() {
    var me = this;
    me.props.root.toggleSubComp(me.props.rowData);
  };

  Row.prototype.toggleExpanded = function toggleExpanded(e) {
    e.stopPropagation();
    var me = this;
    var rowData = me.props.rowData;

    me.props.root.toggleTreeExpanded(rowData);
  };

  Row.prototype.renderSubComp = function renderSubComp() {
    var props = this.props;

    if (props.renderModel === 'tree') {
      return false;
    }
    if (props.subComp) {
      if (props.rowData.showSubComp) {
        var subComp = React.cloneElement(props.subComp, {
          passedData: this.props.rowData,
          parentHasCheckbox: !!this.props.rowSelection,
          parentHasCheck: !!this.props.rowSelection
        });
        return React.createElement(
          'div',
          { className: 'kuma-uxtable-subrow' },
          subComp
        );
      }
      return false;
    } else if (props.renderSubComp) {
      var _subComp = props.renderSubComp(deepcopy(props.rowData));
      if (_subComp && props.rowData.showSubComp) {
        return React.createElement(
          'div',
          { className: 'kuma-uxtable-subrow' },
          _subComp
        );
      }
      return false;
    }
    return false;
  };

  Row.prototype.renderChild = function renderChild() {
    var props = this.props;
    var me = this;
    var children = [];

    if (props.renderModel !== 'tree') {
      return children;
    }
    if (props.rowData.data) {
      props.rowData.data.forEach(function (node, index) {
        var renderProps = assign({}, props, {
          level: me.props.level + 1,
          index: index,
          dataIndex: (me.props.dataIndex ? me.props.dataIndex : me.props.index) + '-' + index,
          rowData: node,
          rowIndex: node.jsxid,
          key: node.jsxid,
          showSubComp: false,
          visible: props.expandedKeys.indexOf(props.rowData.jsxid) !== -1
        });
        children.push(React.createElement(Row, renderProps));
      });

      var renderProps = {
        key: 'treeRow' + this.props.rowData.jsxid,
        className: 'kuma-uxtable-tree-row'
      };

      children = React.createElement(
        'ul',
        renderProps,
        children
      );
    }

    return children;
  };

  Row.prototype.renderExpandIcon = function renderExpandIcon(rowIndex) {
    var expandCollapseIcon = void 0;
    var _expandIconClass = void 0;
    var props = this.props;

    if (props.renderModel !== 'tree') {
      return false;
    }

    if (props.rowData.data) {
      _expandIconClass = {
        'kuma-icon': true,
        'kuma-icon-triangle-right': true,
        expanded: props.expandedKeys.indexOf(props.rowData.jsxid) !== -1
      };
      expandCollapseIcon = React.createElement(
        'span',
        {
          className: 'kuma-uxtable-expand-icon',
          'data-type': props.fixedColumn,
          'data-index': rowIndex,
          onClick: this.toggleExpanded.bind(this)
        },
        React.createElement('i', { className: classnames(_expandIconClass) })
      );
    } else {
      expandCollapseIcon = React.createElement('span', { className: 'kuma-uxtable-emptyicon' });
    }
    return expandCollapseIcon;
  };

  Row.prototype.renderIndent = function renderIndent() {
    var indents = [];
    if (this.props.renderModel === 'tree') {
      for (var i = 0; i < this.props.level - 1; i++) {
        var renderProps = {
          className: 'indent',
          key: 'indent' + i
        };
        indents.push(React.createElement('span', renderProps));
      }
    }

    return indents;
  };

  Row.prototype.renderTreeRowSelector = function renderTreeRowSelector() {
    if (this.props.renderModel !== 'tree' || !this.props.checkboxColumnKey) {
      return false;
    }
    var me = this;
    var _me$props = me.props,
        rowData = _me$props.rowData,
        checkboxColumnKey = _me$props.checkboxColumnKey,
        prefixCls = _me$props.prefixCls;

    var isChecked = rowData[checkboxColumnKey];
    var isHalfChecked = false;
    if (!isChecked) {
      isHalfChecked = util.isRowHalfChecked(rowData, checkboxColumnKey);
    }
    return React.createElement(CheckBox, {
      checked: isChecked,
      halfChecked: isHalfChecked,
      className: prefixCls + '-tree-selector',
      onChange: me.handleTreeCheckChange.bind(me)
    });
  };

  Row.prototype.render = function render() {
    var _classnames,
        _this2 = this;

    var props = this.props;
    var _columns = [];
    var _style = {};
    var _data = props.data;
    var me = this;
    var otherCls = props.addRowClassName(_data[props.index]);

    if (!this.props.visible) {
      _style = {
        display: 'none'
      };
    }

    props.columns.forEach(function (column) {
      if ('group' in column) {
        _columns = _columns.concat(column.columns);
      } else {
        _columns.push(column);
      }
    });

    var firstVisableColumn = 0;

    return React.createElement(
      'li',
      {
        className: classnames((_classnames = {}, _defineProperty(_classnames, this.props.prefixCls, true), _defineProperty(_classnames, this.props.prefixCls + '-hover', props.isHover), _defineProperty(_classnames, otherCls, !!otherCls), _defineProperty(_classnames, 'even', props.index % 2 === 1), _defineProperty(_classnames, 'last', props.last), _classnames)),
        style: _style,
        onClick: function onClick() {
          _this2.handleClick(props.rowData);
        },
        onDoubleClick: function onDoubleClick() {
          _this2.handleDoubleClick(props.rowData);
        },
        onMouseEnter: function onMouseEnter() {
          _this2.handleMouseEnter();
        },
        onMouseLeave: function onMouseLeave() {
          _this2.handleMouseLeave();
        }
      },
      React.createElement(
        'div',
        { className: this.props.prefixCls + '-cells' },
        _columns.map(function (item, index) {
          var rowSelectorInTreeMode = ['checkboxSelector', 'radioSelector'].indexOf(item.type) !== -1 && props.renderModel === 'tree';
          if (item.hidden || rowSelectorInTreeMode) {
            return null;
          }
          firstVisableColumn += 1;
          var hasSubComp = !!props.subComp;
          if (!hasSubComp) {
            hasSubComp = props.renderSubComp ? !!props.renderSubComp(deepcopy(props.rowData)) : false;
          }
          var renderProps = {
            key: 'cell' + index,
            column: item,
            root: props.root,
            locale: props.locale,
            rowData: props.rowData,
            rowIndex: props.rowIndex,
            index: props.index,
            changeSelected: props.changeSelected,
            rowSelection: props.rowSelection,
            actions: props.actions,
            mode: props.mode,
            handleDataChange: props.handleDataChange,
            attachCellField: props.attachCellField,
            detachCellField: props.detachCellField,
            last: index === _columns.length - 1,
            cellIndex: index,
            hasSubComp: hasSubComp,
            showSubCompCallback: me.showSubCompFunc.bind(me)
          };

          if (firstVisableColumn === 1) {
            return React.createElement(
              Cell,
              renderProps,
              me.renderIndent(),
              me.renderExpandIcon(props.rowIndex),
              me.renderTreeRowSelector()
            );
          }
          // if have vertical data structure, how to process it
          return React.createElement(Cell, renderProps);
        })
      ),
      me.renderChild(),
      this.renderSubComp()
    );
  };

  return Row;
}(React.Component);

Row.propTypes = {
  prefixCls: React.PropTypes.string,
  renderModel: React.PropTypes.string,
  checkboxColumnKey: React.PropTypes.string,
  root: React.PropTypes.any,
  rowData: React.PropTypes.object,
  rowSelection: React.PropTypes.object,
  showSubComp: React.PropTypes.bool,
  last: React.PropTypes.bool,
  visible: React.PropTypes.bool,
  isHover: React.PropTypes.bool,
  level: React.PropTypes.number,
  levels: React.PropTypes.number,
  onClick: React.PropTypes.func,
  onMouseEnter: React.PropTypes.func,
  index: React.PropTypes.number
};

Row.defaultProps = {
  prefixCls: 'kuma-uxtable-row',
  showSubComp: false,
  onClick: function onClick() {},
  onMouseEnter: function onMouseEnter() {}
};

exports["default"] = Row;
module.exports = exports['default'];