'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

/**
 * Table Component for uxcore
 * @author zhouquan.yezq
 *
 * Copyright 2014-2015, UXCore Team, Alinw.
 * All rights reserved.
 */

/* eslint-disable react/sort-comp */

var CellField = require('uxcore-cell-field');
var Pagination = require('uxcore-pagination');
var Const = require('uxcore-const');
var assign = require('object-assign');
var deepcopy = require('lodash/cloneDeep');
var upperFirst = require('lodash/upperFirst');
var deepEqual = require('lodash/isEqual');
var classnames = require('classnames');
var NattyFetch = require('natty-fetch/dist/natty-fetch.pc');
var Promise = require('lie');
var React = require('react');
var Animate = require('uxcore-animate');

var _require = require('rc-util/lib/Dom/class'),
    addClass = _require.addClass,
    removeClass = _require.removeClass;

var Mask = require('./Mask');
var util = require('./util');
var Header = require('./Header');
var Tbody = require('./Tbody');
var ActionBar = require('./ActionBar');
var methods = require('./methods');
var createCellField = require('./createCellField');

var Table = function (_React$Component) {
  _inherits(Table, _React$Component);

  function Table(props) {
    _classCallCheck(this, Table);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    _this.uid = 0;
    _this.fields = {};
    _this.copyData = deepcopy(_this.props.jsxdata);
    _this.hasFixed = util.hasFixColumn(props);
    _this.state = {
      data: _this.addValuesInData(_this.copyData), // checkbox 内部交互
      columns: _this.processColumn(), // column 内部交互
      showMask: props.showMask, // fetchData 时的内部状态改变
      pageSize: props.pageSize, // pagination 相关
      currentPage: props.currentPage, // pagination 相关
      activeColumn: null,
      searchTxt: '',
      expandedKeys: []
    };
    _this.handleBodyScroll = _this.handleBodyScroll.bind(_this);
    _this.handleHeaderScroll = _this.handleHeaderScroll.bind(_this);
    _this.changeSelected = _this.changeSelected.bind(_this);
    _this.handleDataChange = _this.handleDataChange.bind(_this);
    _this.attachCellField = _this.attachCellField.bind(_this);
    _this.detachCellField = _this.detachCellField.bind(_this);
    _this.selectAll = _this.selectAll.bind(_this);
    _this.handleOrderColumnCB = _this.handleOrderColumnCB.bind(_this);
    _this.handleColumnPickerChange = _this.handleColumnPickerChange.bind(_this);
    _this.handleActionBarSearch = _this.handleActionBarSearch.bind(_this);
    return _this;
  }

  Table.prototype.componentDidMount = function componentDidMount() {
    var _this2 = this;

    var me = this;
    if (!!me.state.data && !!me.state.data.datas) {
      console.warn('Table: "content.data" rather than "content.datas" is recommended, ' + 'the support for "content.datas" will be end from ver. 1.5.0');
    }
    if (me.props.subComp) {
      console.warn('Table: subComp is deprecated, use renderSubComp instead.');
    }
    if (me.props.renderSubComp && this.hasFixed) {
      console.error('Table: subComp cannot be rendered if fixed column exists, remove fixed column or props.renderSubComp');
    }
    if (this.props.fetchDataOnMount) {
      this.fetchData(undefined, undefined, function () {
        _this2.checkBodyHScroll();
      });
    }
    Object.keys(methods).forEach(function (key) {
      me[key] = methods[key].bind(me);
    });
  };

  Table.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    var me = this;
    var newData = {};
    if (nextProps.jsxdata && !deepEqual(nextProps.jsxdata, this.copyData)) {
      // Data has changed, so uid which is used to mark the data should be reset.
      me.uid = 0;
      me.fetchData('dataChange', nextProps);
    }
    if (nextProps.pageSize !== me.props.pageSize) {
      newData.pageSize = nextProps.pageSize;
    }
    if (nextProps.currentPage !== me.props.currentPage) {
      newData.currentPage = nextProps.currentPage;
    }
    if (!!nextProps.jsxcolumns && !deepEqual(nextProps.jsxcolumns, me.props.jsxcolumns)) {
      newData.columns = me.processColumn(nextProps);
      this.hasFixed = util.hasFixColumn(nextProps);
    }
    if (nextProps.showMask !== me.props.showMask) {
      newData.showMask = nextProps.showMask;
    }
    if (nextProps.fetchUrl !== me.props.fetchUrl || !deepEqual(nextProps.fetchParams, me.props.fetchParams)) {
      me.fetchData('propsChange', nextProps);
    }
    me.setState(newData);
  };

  Table.prototype.renderTbody = function renderTbody(renderBodyProps, bodyHeight, fixedColumn) {
    var isFixedTable = ['fixed', 'rightFixed'].indexOf(fixedColumn) !== -1;
    var scrollBarWidth = util.measureScrollbar();
    return React.createElement(
      'div',
      {
        className: classnames('kuma-uxtable-body-wrapper', {
          'kuma-uxtable-fixed-body-wrapper': isFixedTable
        }),
        style: {
          height: isFixedTable ? bodyHeight - scrollBarWidth : bodyHeight
        }
      },
      React.createElement(Tbody, _extends({}, renderBodyProps, {
        fixedColumn: fixedColumn,
        onScroll: this.handleBodyScroll,
        ref: util.saveRef('body' + upperFirst(fixedColumn), this)
      })),
      !isFixedTable ? React.createElement(
        Animate,
        { showProp: 'visible', transitionName: 'tableMaskFade' },
        React.createElement(Mask, { visible: this.state.showMask, text: this.props.loadingText })
      ) : null
    );
  };

  Table.prototype.renderHeader = function renderHeader(renderHeaderProps, fixedColumn) {
    if (!this.props.showHeader) {
      return null;
    }
    return React.createElement(
      'div',
      { className: 'kuma-uxtable-header-wrapper' },
      React.createElement(Header, _extends({}, renderHeaderProps, {
        fixedColumn: fixedColumn,
        ref: util.saveRef('header' + upperFirst(fixedColumn), this),
        onScroll: this.handleHeaderScroll
      }))
    );
  };

  /**
   * get Query Object by combining data from searchBar, column order, pagination
   * and fetchParams.
   * @param from {string} used in props.beforeFetch
   */

  Table.prototype.getQueryObj = function getQueryObj(from, props) {
    var me = this;
    var queryObj = {};
    if (props.passedData) {
      var queryKeys = props.queryKeys;
      if (!queryKeys) {
        queryObj = props.passedData;
      } else {
        queryKeys.forEach(function (key) {
          if (props.passedData[key] !== undefined) {
            queryObj[key] = props.passedData[key];
          }
        });
      }
    }

    // pagination
    queryObj = assign({}, queryObj, {
      pageSize: me.state.pageSize,
      currentPage: me.state.currentPage
    });

    // column order
    var activeColumn = me.state.activeColumn;
    var orderType = me.state.orderType;
    if (activeColumn) {
      queryObj = assign({}, queryObj, {
        orderColumn: activeColumn.dataKey
      });
      if (orderType && orderType !== 'none') {
        queryObj.orderType = orderType;
      }
    }

    // search query
    var searchTxt = me.state.searchTxt;
    if (searchTxt) {
      queryObj = assign({}, queryObj, {
        searchTxt: searchTxt
      });
    }

    // fetchParams has the top priority
    if (props.fetchParams) {
      queryObj = assign({}, queryObj, props.fetchParams);
    }

    return props.beforeFetch(queryObj, from);
  };

  Table.prototype.onPageChange = function onPageChange(current) {
    var me = this;
    me.setState({
      currentPage: current
    }, function () {
      me.fetchData('pagination');
    });
  };

  Table.prototype.getDom = function getDom() {
    return this.root;
  };

  Table.prototype.getPager = function getPager() {
    return this.pager;
  };

  Table.prototype.getCheckStatus = function getCheckStatus(data) {
    var me = this;
    var rowSelection = me.props.rowSelection;

    var column = me.checkboxColumn;
    if (!column || data.length === 0) {
      return false;
    }
    var checkboxColumnKey = me.checkboxColumnKey;
    var isAllDisabled = true;
    var isHalfChecked = false;
    var checkedColumn = 0;
    var enabledColumn = 0;
    for (var i = 0; i < data.length; i++) {
      var item = data[i];
      if (!column.disable && !(column.isDisable && column.isDisable(item)) && !((typeof rowSelection === 'undefined' ? 'undefined' : _typeof(rowSelection)) === 'object' && rowSelection.isDisabled && rowSelection.isDisabled(item))) {
        isAllDisabled = false;
        enabledColumn += 1;
        if (item[checkboxColumnKey]) {
          isHalfChecked = true;
          checkedColumn += 1;
        }
      }
    }
    var isAllChecked = enabledColumn ? checkedColumn === enabledColumn : false;
    return { isAllChecked: isAllChecked, isAllDisabled: isAllDisabled, isHalfChecked: isAllChecked ? false : isHalfChecked };
  };

  /**
   * change the checkboxColumnKey of data, passed to the Row
   * @param checked {boolean} tree checkbox status
   * @param dataIndex {string} like `1-2-3` means the position of the Row in data
   */


  Table.prototype.changeTreeSelected = function changeTreeSelected(checked, dataIndex) {
    var me = this;
    var currentLevel = dataIndex.toString().split('-');
    var levelDepth = currentLevel.length;
    var data = deepcopy(me.state.data);
    var current = data.data;
    // record each tree node for reverse recursion.
    var treeMap = [];
    for (var i = 0; i < levelDepth - 1; i++) {
      treeMap[i] = current;
      current = current[currentLevel[i]].data;
    }
    // check/uncheck current row and all its children
    current = current[currentLevel[levelDepth - 1]];
    current[me.checkboxColumnKey] = checked;
    util.changeValueR(current, me.checkboxColumnKey, checked);

    // reverse recursion, check/uncheck parents by its children.
    for (var _i = treeMap.length - 1; _i >= 0; _i--) {
      treeMap[_i][currentLevel[_i]][me.checkboxColumnKey] = treeMap[_i][currentLevel[_i]].data.every(function (item) {
        return item[me.checkboxColumnKey] === true;
      });
    }

    me.setState({
      data: data
    }, function () {
      var selectedRows = util.getAllSelectedRows(deepcopy(data), me.checkboxColumnKey);
      if (me.props.rowSelection && me.props.rowSelection.onSelect) {
        me.props.rowSelection.onSelect(checked, current, selectedRows);
      }
    });
  };

  Table.prototype.selectAll = function selectAll(checked) {
    var me = this;
    var content = deepcopy(me.state.data);
    var data = content.datas || content.data;
    var rowSelection = me.props.rowSelection;

    var selectedRows = [];
    for (var i = 0; i < data.length; i++) {
      var column = me.checkboxColumn;
      var key = me.checkboxColumnKey;
      var item = data[i];
      if ((!('isDisable' in column) || !column.isDisable(item)) && !column.disable && !((typeof rowSelection === 'undefined' ? 'undefined' : _typeof(rowSelection)) === 'object' && rowSelection.isDisabled && rowSelection.isDisabled(item))) {
        item[key] = checked;
        selectedRows.push(item);
      }
    }

    if (!!rowSelection && !!rowSelection.onSelectAll) {
      rowSelection.onSelectAll.apply(null, [checked, checked ? selectedRows : []]);
    }
    me.setState({
      data: content
    });
  };

  Table.prototype.handleRowHover = function handleRowHover(index, isEnter) {
    var _this3 = this;

    if (!isEnter) {
      this.rowHoverTimer = setTimeout(function () {
        _this3.setState({
          currentHoverRow: -1
        });
      }, 100);
    } else {
      if (this.rowHoverTimer) {
        clearTimeout(this.rowHoverTimer);
        this.rowHoverTimer = null;
      }
      this.setState({
        currentHoverRow: index
      });
    }
  };

  Table.prototype.handleShowSizeChange = function handleShowSizeChange(current, pageSize) {
    var me = this;
    me.setState({
      currentPage: current,
      pageSize: pageSize
    }, function () {
      me.fetchData('pagination');
    });
  };

  Table.prototype.handleColumnPickerChange = function handleColumnPickerChange(checkedKeys, groupName) {
    var columns = deepcopy(this.state.columns);
    var notRenderColumns = ['jsxchecked', 'jsxtreeIcon', 'jsxwhite'];
    var commonGroupName = util.getConsts().commonGroup;
    for (var i = 0; i < columns.length; i++) {
      var item = columns[i];
      var isGroup = {}.hasOwnProperty.call(item, 'columns') && _typeof(item.columns) === 'object';
      // current column is a group and groupName is right
      if (isGroup && item.group === groupName) {
        for (var j = 0; j < item.columns.length; j++) {
          var ele = item.columns[j];
          if (checkedKeys.indexOf(ele.dataKey) !== -1) {
            ele.hidden = false;
          } else {
            ele.hidden = true;
          }
        }
        break;
      } else if (groupName === commonGroupName) {
        // current column is common group
        if (checkedKeys.indexOf(item.dataKey) !== -1 || notRenderColumns.indexOf(item.dataKey) !== -1) {
          item.hidden = false;
        } else {
          item.hidden = true;
        }
      }
    }

    var selectedKeys = util.getSelectedKeys(columns);

    if (selectedKeys.length === 0) {
      return;
    }

    this.setState({
      columns: columns
    });
  };

  Table.prototype.handleBodyScroll = function handleBodyScroll(scrollLeft, scrollTop, column) {
    var me = this;
    var headerNode = me.headerScroll;
    if (scrollLeft !== undefined && column === 'scroll') {
      headerNode.getDom().scrollLeft = scrollLeft;
    }
    if (scrollTop !== undefined && this.hasFixed) {
      var columnType = ['fixed', 'rightFixed', 'scroll'];
      var columnToScroll = columnType.filter(function (item) {
        return item !== column;
      });
      columnToScroll.forEach(function (item) {
        var instance = me['body' + upperFirst(item)];
        if (instance) {
          instance.getDom().scrollTop = scrollTop;
        }
      });
    }
    me.checkBodyHScroll(scrollLeft);
  };

  Table.prototype.handleHeaderScroll = function handleHeaderScroll(scrollLeft) {
    var me = this;
    var bodyNode = me.bodyScroll;
    if (scrollLeft !== undefined) {
      bodyNode.getDom().scrollLeft = scrollLeft;
    }
  };

  Table.prototype.checkBodyHScroll = function checkBodyHScroll(scrollLeft) {
    if (!this.hasFixed) {
      return false;
    }
    var node = this.bodyScroll.getDom();
    var wrapperScrollLeft = scrollLeft || node.scrollLeft;
    if (this.hasFixed.hasLeft) {
      if (wrapperScrollLeft > 0) {
        addClass(this.fixedTable, 'has-scroll');
      } else {
        removeClass(this.fixedTable, 'has-scroll');
      }
    }
    if (this.hasFixed.hasRight) {
      var wrapperWidth = node.clientWidth;
      var bodyWidth = node.children[0].clientWidth;
      if (wrapperScrollLeft + wrapperWidth + 3 < bodyWidth) {
        addClass(this.rightFixedTable, 'end-of-scroll');
      } else {
        removeClass(this.rightFixedTable, 'end-of-scroll');
      }
    }
    return false;
  };

  Table.prototype.handleOrderColumnCB = function handleOrderColumnCB(type, column) {
    var me = this;
    me.setState({
      activeColumn: column,
      orderType: type
    }, function () {
      me.fetchData('order');
    });
  };

  Table.prototype.handleActionBarSearch = function handleActionBarSearch(value) {
    var me = this;
    this.setState({
      searchTxt: value
    }, function () {
      me.fetchData('search');
    });
  };

  Table.prototype.processColumn = function processColumn(props) {
    var actualProps = props || this.props;

    var me = this;
    var columns = deepcopy(actualProps.jsxcolumns);
    var hasCheckboxColumn = false;

    for (var i = 0; i < columns.length; i++) {
      var item = columns[i];
      // only one rowSelector can be rendered in Table.
      if (item.type === 'checkbox' || item.type === 'radioSelector' || item.type === 'checkboxSelector') {
        if (item.type === 'checkbox') {
          console.warn("rowSelector using 'type: checkbox' is deprecated," + " use 'type: checkboxSelector' instead.");
        }
        hasCheckboxColumn = true;
        me.checkboxColumn = item;
        me.checkboxColumnKey = item.dataKey;
        item.width = item.width || (/kuma-uxtable-border-line/.test(actualProps.className) ? 40 : 32);
        item.align = item.align || 'left';
      }
    }

    // filter the column which has a dataKey 'jsxchecked' & 'jsxtreeIcon'

    columns = columns.filter(function (item) {
      return item.dataKey !== 'jsxchecked' && item.dataKey !== 'jsxtreeIcon';
    });

    if (!!actualProps.rowSelection && !hasCheckboxColumn) {
      me.checkboxColumn = {
        dataKey: 'jsxchecked',
        width: 46,
        type: actualProps.rowSelector,
        align: 'right'
      };
      me.checkboxColumnKey = 'jsxchecked';
      columns = [me.checkboxColumn].concat(columns);
    } else if (actualProps.parentHasCheckbox) {
      // no rowSelection but has parentHasCheckbox, render placeholder
      columns = [{
        dataKey: 'jsxwhite',
        width: 46,
        type: 'empty'
      }].concat(columns);
    }
    if ((actualProps.subComp || actualProps.renderSubComp) && actualProps.renderModel !== 'tree' && !this.hasFixed) {
      columns = [{
        dataKey: 'jsxtreeIcon',
        width: 36,
        type: 'treeIcon'
      }].concat(columns);
    } else if (actualProps.passedData) {
      // no subComp but has passedData, means sub mode, parent should has tree icon,
      // render tree icon placeholder
      columns = [{
        dataKey: 'jsxwhite',
        width: 34,
        type: 'empty'
      }].concat(columns);
    }
    return columns;
  };

  /**
   * fetch Data via Ajax
   * @param from {string} tell fetchData where it is invoked, the param will be
   * passed to props.beforeFetch in order to help the user.
   */

  Table.prototype.fetchData = function fetchData(from, nextProps, cb) {
    var me = this;
    var props = nextProps || this.props;
    // reset uid cause table data has changed
    me.uid = 0;

    // fetchUrl has the top priority.
    if (props.fetchUrl) {
      me.fetchRemoteData(from, props, cb);
    } else if (props.passedData) {
      me.fetchPassedData(props, cb);
    } else if (props.jsxdata) {
      me.fetchLocalData(from, props, cb);
    } else {
      // default will create one row
      var data = {
        data: [{
          jsxid: me.uid,
          __mode__: Const.MODE.EDIT
        }],
        currentPage: 1,
        totalCount: 0
      };
      me.uid += 1;
      me.data = data;
      me.setState({
        data: data
      });
    }
  };

  Table.prototype.fetchRemoteData = function fetchRemoteData(from, props) {
    var cb = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function () {};

    var me = this;
    if (me.request) {
      me.request.abort();
    }
    if (!me.state.showMask) {
      me.setState({
        showMask: true
      });
    }
    var isJsonp = props.isJsonp === undefined ? /\.jsonp/.test(props.fetchUrl) : props.isJsonp;
    me.request = NattyFetch.create({
      url: props.fetchUrl,
      data: me.getQueryObj(from, props),
      fit: props.fitResponse,
      jsonp: isJsonp,
      Promise: Promise
    });

    me.request().then(function (content) {
      var processedData = me.addValuesInData(props.processData(deepcopy(content)));
      var updateObj = {
        data: processedData,
        showMask: false,
        expandedKeys: util.getDefaultExpandedKeys(processedData.data, props.levels)
      };
      if (processedData.currentPage !== undefined) {
        updateObj.currentPage = processedData.currentPage;
      }
      me.data = deepcopy(processedData);
      me.setState(updateObj, function () {
        cb();
      });
    })["catch"](function (err) {
      props.onFetchError(err);
      me.setState({
        data: {
          data: []
        },
        showMask: false
      });
    });
  };

  Table.prototype.fetchPassedData = function fetchPassedData(props) {
    var cb = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {};

    console.warn('props subComp is deprecated, use renderSubComp instead.');
    var me = this;
    if (!props.queryKeys) {
      var data = me.addValuesInData(props.processData(deepcopy(props.passedData)));
      me.setState({
        data: data
      });
      me.data = deepcopy(data);
    } else {
      var _data2 = {};
      props.queryKeys.forEach(function (key) {
        if (props.passedData[key] !== undefined) {
          _data2[key] = props.passedData[key];
        }
      });
      var processedData = me.addValuesInData(props.processData(deepcopy(_data2)));
      me.data = deepcopy(processedData);
      me.setState({
        data: processedData
      }, function () {
        cb();
      });
    }
  };

  Table.prototype.fetchLocalData = function fetchLocalData(from, props) {
    var cb = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function () {};

    var me = this;
    if (['pagination', 'order', 'search'].indexOf(from) !== -1) {
      if (from === 'pagination' && props.onPagerChange) {
        props.onPagerChange(me.state.currentPage, me.state.pageSize);
      }

      if (from === 'order' && props.onOrder) {
        props.onOrder(me.state.activeColumn, me.state.orderType);
      }

      if (from === 'search' && props.onSearch) {
        props.onSearch(me.state.searchTxt);
      }
    } else {
      this.copyData = deepcopy(props.jsxdata);
      var data = this.addValuesInData(this.copyData);
      var currentPage = data && data.currentPage || this.state.currentPage;
      me.data = deepcopy(data);
      me.setState({
        data: data,
        currentPage: currentPage,
        expandedKeys: util.getDefaultExpandedKeys(data.data, props.levels)
      }, function () {
        cb();
      });
    }
  };

  /**
  * cancel the CellField when it is unmounted.
  * @param field {element} the cell field to be canceled.
  */

  Table.prototype.detachCellField = function detachCellField(name) {
    delete this.fields[name];
  };

  /**
   * register CellField to Table for the global validation
   * @param field {element} the cell field to be registered
   */

  Table.prototype.attachCellField = function attachCellField(validate, name) {
    var me = this;
    if (!name) {
      console.error('Table: dataKey can not be undefined, check the column config');
    } else {
      me.fields[name] = validate;
    }
  };

  /**
   * For inline edit
   * receive changes from cell field and change state.data
   * inform users of the change with dataKey & pass
   */

  Table.prototype.handleDataChange = function handleDataChange(obj) {
    var me = this;
    var jsxid = obj.jsxid,
        column = obj.column,
        value = obj.value,
        text = obj.text,
        pass = obj.pass;

    var dataKey = column.dataKey;
    var editKey = column.editKey || dataKey;
    var data = deepcopy(me.state.data);
    var changedData = {};
    for (var i = 0; i < data.data.length; i++) {
      if (data.data[i].jsxid === jsxid) {
        data.data[i][dataKey] = text;
        data.data[i][editKey] = value;
        changedData = data.data[i];
      }
    }

    me.setState({
      data: data
    }, function () {
      me.props.onChange({
        data: me.state.data,
        editKey: editKey,
        dataKey: dataKey,
        changedData: changedData,
        pass: pass
      });
    });
  };

  /**
   * change SelectedRows data via checkbox, this function will pass to the Cell
   * @param checked {boolean} the checkbox status
   * @param rowIndex {number} the row Index
   * @param fromMount {boolean} onSelect is called from cell Mount is not expected.
   */

  Table.prototype.changeSelected = function changeSelected(checked, rowIndex, fromMount) {
    var me = this;
    var content = deepcopy(this.state.data);
    var _data = content.datas || content.data;

    if (me.checkboxColumn.type === 'radioSelector') {
      for (var i = 0; i < _data.length; i++) {
        var item = _data[i];
        if (item.jsxid === rowIndex) {
          item[me.checkboxColumnKey] = checked;
        } else if (item[me.checkboxColumnKey]) {
          item[me.checkboxColumnKey] = false;
        }
      }
    } else {
      for (var _i2 = 0; _i2 < _data.length; _i2++) {
        var _item = _data[_i2];
        if (_item.jsxid === rowIndex) {
          _item[me.checkboxColumnKey] = checked;
          break;
        }
      }
    }

    me.setState({
      data: content
    }, function () {
      if (!fromMount) {
        var data = me.state.data.datas || me.state.data.data;
        var selectedRows = data.filter(function (item) {
          return item[me.checkboxColumnKey] === true;
        });
        if (me.props.rowSelection && me.props.rowSelection.onSelect) {
          me.props.rowSelection.onSelect(checked, data[rowIndex], selectedRows);
        }
      }
    });
  };

  Table.prototype.renderPager = function renderPager() {
    var me = this;
    var _me$state = me.state,
        data = _me$state.data,
        currentPage = _me$state.currentPage,
        pageSize = _me$state.pageSize;
    var _me$props = me.props,
        showPagerTotal = _me$props.showPagerTotal,
        showPager = _me$props.showPager,
        locale = _me$props.locale,
        pagerSizeOptions = _me$props.pagerSizeOptions,
        isMiniPager = _me$props.isMiniPager,
        showPagerSizeChanger = _me$props.showPagerSizeChanger;

    if (showPager && data && data.totalCount) {
      return React.createElement(
        'div',
        { className: 'kuma-uxtable-page' },
        React.createElement(Pagination, {
          className: classnames({
            mini: isMiniPager
          }),
          ref: util.saveRef('pager', me),
          locale: locale,
          showSizeChanger: showPagerSizeChanger,
          showTotal: showPagerTotal,
          total: data.totalCount,
          onShowSizeChange: me.handleShowSizeChange.bind(me),
          onChange: me.onPageChange.bind(me),
          current: currentPage,
          pageSize: pageSize,
          sizeOptions: pagerSizeOptions
        })
      );
    }
    return null;
  };

  Table.prototype.renderMainTable = function renderMainTable(renderHeaderProps, renderBodyProps, bodyHeight) {
    var prefixCls = this.props.prefixCls;

    return React.createElement(
      'div',
      { className: prefixCls + '-main-table', ref: util.saveRef('mainTable', this) },
      this.renderHeader(renderHeaderProps, 'scroll'),
      this.renderTbody(renderBodyProps, bodyHeight, 'scroll')
    );
  };

  Table.prototype.renderLeftFixedTable = function renderLeftFixedTable(renderHeaderProps, renderBodyProps, bodyHeight) {
    if (!this.hasFixed || !this.hasFixed.hasLeft || !renderBodyProps.data || !renderBodyProps.data.length) {
      return null;
    }
    var prefixCls = this.props.prefixCls;

    return React.createElement(
      'div',
      { className: prefixCls + '-left-fixed-table', ref: util.saveRef('fixedTable', this) },
      this.renderHeader(renderHeaderProps, 'fixed'),
      this.renderTbody(renderBodyProps, bodyHeight, 'fixed')
    );
  };

  Table.prototype.renderRightFixedTable = function renderRightFixedTable(renderHeaderProps, renderBodyProps, bodyHeight) {
    if (!this.hasFixed || !this.hasFixed.hasRight || !renderBodyProps.data || !renderBodyProps.data.length) {
      return null;
    }
    var prefixCls = this.props.prefixCls;

    return React.createElement(
      'div',
      { className: prefixCls + '-right-fixed-table', ref: util.saveRef('rightFixedTable', this) },
      this.renderHeader(renderHeaderProps, 'rightFixed'),
      this.renderTbody(renderBodyProps, bodyHeight, 'rightFixed')
    );
  };

  Table.prototype.render = function render() {
    var _classnames;

    var me = this;
    var props = this.props,
        state = this.state;

    var bodyHeight = void 0;
    // if table is in sub mode, people always want to align the parent
    // and the sub table, so width should not be cared.
    var headerHeight = props.headerHeight;


    var data = state.data ? state.data.datas || state.data.data : [];
    var checkStatus = me.getCheckStatus(data);

    var style = {
      width: props.passedData ? 'auto' : props.width,
      height: props.height
    };
    var actionBarHeight = props.actionBar || props.showSearch ? props.actionBarHeight : 0;
    var pagerHeight = props.showPager && this.state.data && this.state.data.totalCount ? 42 : 0;

    // decide whether the table has column groups
    var hasGroup = false;
    for (var i = 0; i < this.state.columns.length; i++) {
      if ('group' in this.state.columns[i]) {
        hasGroup = true;
        break;
      }
    }
    if (props.height === 'auto' || props.height === '100%') {
      bodyHeight = props.height;
    } else {
      bodyHeight = parseInt(props.height, 10) - (headerHeight || (hasGroup ? 100 : 50)) - actionBarHeight - pagerHeight;
    }
    var renderBodyProps = {
      columns: state.columns,
      mask: state.showMask,
      expandedKeys: state.expandedKeys,
      currentHoverRow: state.currentHoverRow,
      data: data,
      bodyHeight: bodyHeight,
      rowSelection: props.rowSelection,
      addRowClassName: props.addRowClassName,
      locale: props.locale,
      emptyText: props.emptyText,
      renderSubComp: this.hasFixed ? null : props.renderSubComp,
      rowHeight: props.rowHeight,
      loadingText: props.loadingText,
      checkboxColumnKey: me.checkboxColumnKey,
      height: bodyHeight,
      width: props.width,
      mode: props.mode,
      levels: props.levels,
      root: this,
      renderModel: props.renderModel,
      changeSelected: this.changeSelected,
      handleDataChange: this.handleDataChange,
      attachCellField: this.attachCellField,
      detachCellField: this.detachCellField,
      key: 'grid-body'
    };
    var renderHeaderProps = {
      columns: state.columns,
      activeColumn: state.activeColumn,
      orderType: state.orderType,
      checkboxColumnKey: me.checkboxColumnKey,
      showHeaderBorder: props.showHeaderBorder,
      headerHeight: props.headerHeight,
      renderModel: props.renderModel,
      width: props.width,
      mode: props.mode,
      checkStatus: checkStatus,
      selectAll: this.selectAll,
      orderColumnCB: this.handleOrderColumnCB,
      key: 'grid-header'
    };

    var actionBar = void 0;

    if (props.actionBar || props.linkBar || props.showSearch || props.showColumnPicker) {
      var renderActionProps = {
        actionBarConfig: this.props.actionBar,
        showColumnPicker: this.props.showColumnPicker,
        locale: this.props.locale,
        linkBar: this.props.linkBar,
        checkboxColumnKey: me.checkboxColumnKey,
        showSearch: this.props.showSearch,
        searchBarPlaceholder: this.props.searchBarPlaceholder,
        columns: state.columns,
        width: props.width,
        onSearch: this.handleActionBarSearch,
        handleColumnPickerChange: this.handleColumnPickerChange,
        key: 'grid-actionbar'
      };
      actionBar = React.createElement(ActionBar, renderActionProps);
    }

    return React.createElement(
      'div',
      {
        className: classnames((_classnames = {}, _defineProperty(_classnames, props.className, !!props.className), _defineProperty(_classnames, props.prefixCls, true), _defineProperty(_classnames, 'kuma-subgrid-mode', !!props.passedData), _defineProperty(_classnames, props.prefixCls + '-tree-mode', props.renderModel === 'tree'), _classnames)),
        style: style,
        ref: util.saveRef('root', this)
      },
      actionBar,
      React.createElement(
        'div',
        {
          className: 'kuma-uxtable-content',
          style: {
            width: props.passedData ? 'auto' : props.width
          }
        },
        this.renderLeftFixedTable(renderHeaderProps, renderBodyProps, bodyHeight),
        this.renderMainTable(renderHeaderProps, renderBodyProps, bodyHeight),
        this.renderRightFixedTable(renderHeaderProps, renderBodyProps, bodyHeight)
      ),
      this.renderPager()
    );
  };

  // Util Method

  /**
   * add some specific value for each row data which will be used in manipulating data & rendering.
   * used in record API.
   */

  Table.prototype.addJSXIdsForRecord = function addJSXIdsForRecord(obj) {
    var me = this;
    var objAux = deepcopy(obj);
    if (objAux instanceof Array) {
      objAux = objAux.map(function (item) {
        var newItem = deepcopy(item);
        if (newItem.jsxid === undefined || newItem.jsxid == null) {
          me.uid += 1;
          newItem.jsxid = me.uid;
        }
        if (!newItem.__mode__) {
          newItem.__mode__ = Const.MODE.EDIT;
        }
        return newItem;
      });
    } else {
      me.uid += 1;
      objAux.jsxid = me.uid;
    }
    return objAux;
  };

  /**
   * add some specific value for each row data which will be used in manipulating data & rendering.
   * used in method fetchData
   */

  Table.prototype.addValuesInData = function addValuesInData(objAux) {
    if (!objAux || !objAux.datas && !objAux.data) return null;
    var me = this;
    var data = objAux.datas || objAux.data;
    for (var i = 0; i < data.length; i++) {
      var node = data[i];
      node.jsxid = me.uid;
      me.uid += 1;
      node.__mode__ = node.__mode__ || Const.MODE.VIEW;
      node.__treeId__ = objAux.__treeId__ ? objAux.__treeId__ + '-' + i : '' + i;
      me.addValuesInData(node);
    }
    return objAux;
  };

  /**
   * insert some data into this.state.data & this.data
   * @param objAux {Array or Object} datum or data need to be inserted
   */

  Table.prototype.insertRecords = function insertRecords(obj, cb) {
    if ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) !== 'object') return;
    var me = this;
    var objAux = deepcopy(obj);
    if (!(objAux instanceof Array)) {
      objAux = [objAux];
    }
    objAux = me.addJSXIdsForRecord(objAux);
    var content = util.mergeData(me.state.data, objAux);
    me.data = content;
    me.setState({
      data: content
    }, function () {
      if (cb) {
        cb();
      }
    });
  };

  /**
   * update this.state.data using objAux by jsxid
   * @param {object/array} obj
   */


  Table.prototype.updateRecord = function updateRecord(obj, cb) {
    var stateData = deepcopy(this.state.data);

    if (!stateData) {
      return;
    }

    var objAux = deepcopy(obj);
    if (!(objAux instanceof Array)) {
      objAux = [objAux];
    }

    if (stateData.data || stateData.datas) {
      var data = stateData.data || stateData.datas;
      objAux.forEach(function (item) {
        for (var i = 0; i < data.length; i++) {
          if (data[i].jsxid === item.jsxid) {
            data[i] = item;
            break;
          }
        }
      });
      if (stateData.data) {
        stateData.data = data;
      } else if (stateData.datas) {
        stateData.datas = data;
      }
    }
    this.setState({
      data: stateData
    }, function () {
      if (cb) {
        cb();
      }
    });
  };

  /**
   * update this.state.data & this.data using objAux by jsxid
   * @param {objtct/array} objAux
   */

  Table.prototype.syncRecord = function syncRecord(obj, cb) {
    var me = this;
    var data = me.data.data || me.data.datas;
    var objAux = deepcopy(obj);
    if (!(objAux instanceof Array)) {
      objAux = [objAux];
    }
    me.updateRecord(objAux, function () {
      var stateData = deepcopy(me.state.data.data || me.state.data.datas);
      objAux.forEach(function (item) {
        for (var i = 0; i < stateData.length; i++) {
          var element = stateData[i];
          if (element.jsxid === item.jsxid) {
            data[i] = element;
            break;
          }
        }
      });
      if (cb) {
        cb();
      }
    });
  };

  /**
   * remove some items from this.state.data & this.data
   * @param {object/array} obj items to be removed
   */


  Table.prototype.removeRecords = function removeRecords(obj, cb) {
    var me = this;
    var content = deepcopy(me.state.data);
    var data = content.data || content.datas;
    var objAux = deepcopy(obj);
    if (Object.prototype.toString.call(objAux) !== '[object Array]') {
      objAux = [objAux];
    }
    objAux.forEach(function (item) {
      for (var i = 0; i < data.length; i++) {
        var element = data[i];
        if (element.jsxid === item.jsxid) {
          data.splice(i, 1);
          break;
        }
      }
    });
    me.data = content;
    this.setState({
      data: content
    }, function () {
      if (cb) {
        cb();
      }
    });
  };

  return Table;
}(React.Component);

Table.defaultProps = {
  prefixCls: 'kuma-uxtable',
  jsxcolumns: [],
  locale: 'zh-cn',
  showHeader: true,
  width: 'auto',
  height: 'auto',
  mode: Const.MODE.EDIT,
  renderModel: '',
  levels: 0,
  actionBarHeight: 54,
  fetchDataOnMount: true,
  doubleClickToEdit: true,
  rowSelector: 'checkboxSelector',
  showPager: true,
  isMiniPager: false,
  showPagerSizeChanger: true,
  showColumnPicker: false,
  showHeaderBorder: false,
  showPagerTotal: false,
  showMask: false,
  showSearch: false,
  getSavedData: true,
  pageSize: 10,
  pagerSizeOptions: [10, 20, 30, 40],
  rowHeight: 76,
  fetchParams: {},
  currentPage: 1,
  emptyText: React.createElement(
    'div',
    { style: { lineHeight: 2 } },
    '\u6682\u65E0\u6570\u636E'
  ),
  searchBarPlaceholder: '搜索表格内容',
  loadingText: 'loading',
  fitResponse: function fitResponse(response) {
    return {
      content: response.content,
      success: response.success === undefined ? !response.hasError : response.success,
      error: {
        message: response.content || response.errors
      }
    };
  },
  processData: function processData(data) {
    return data;
  },
  beforeFetch: function beforeFetch(obj) {
    return obj;
  },
  onFetchError: function onFetchError(err) {
    if (err && err.stack) {
      console.error(err.stack);
    }
  },
  addRowClassName: function addRowClassName() {},
  onChange: function onChange() {}
};

// http://facebook.github.io/react/docs/reusable-components.html
Table.propTypes = {
  prefixCls: React.PropTypes.string,
  locale: React.PropTypes.string,
  jsxcolumns: React.PropTypes.arrayOf(React.PropTypes.object),
  width: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.number]),
  height: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.number]),
  headerHeight: React.PropTypes.number,
  pageSize: React.PropTypes.number,
  queryKeys: React.PropTypes.array,
  fetchDataOnMount: React.PropTypes.bool,
  doubleClickToEdit: React.PropTypes.bool,
  showColumnPicker: React.PropTypes.bool,
  showPager: React.PropTypes.bool,
  isMiniPager: React.PropTypes.bool,
  showPagerTotal: React.PropTypes.bool,
  pagerSizeOptions: React.PropTypes.array,
  showHeader: React.PropTypes.bool,
  showMask: React.PropTypes.bool,
  showSearch: React.PropTypes.bool,
  searchBarPlaceholder: React.PropTypes.string,
  loadingText: React.PropTypes.string,
  subComp: React.PropTypes.element,
  emptyText: React.PropTypes.oneOfType([React.PropTypes.string, React.PropTypes.element, React.PropTypes.object]),
  jsxdata: React.PropTypes.object,
  fetchUrl: React.PropTypes.string,
  fetchParams: React.PropTypes.object,
  currentPage: React.PropTypes.number,
  rowSelector: React.PropTypes.string,
  actionBar: React.PropTypes.oneOfType([React.PropTypes.array, React.PropTypes.object]),
  linkBar: React.PropTypes.oneOfType([React.PropTypes.array, React.PropTypes.object]),
  fitResponse: React.PropTypes.func,
  processData: React.PropTypes.func,
  beforeFetch: React.PropTypes.func,
  onFetchError: React.PropTypes.func,
  addRowClassName: React.PropTypes.func,
  passedData: React.PropTypes.object,
  getSavedData: React.PropTypes.bool,
  onChange: React.PropTypes.func,
  renderModel: React.PropTypes.string,
  levels: React.PropTypes.number

};

Table.displayName = 'Table';
Table.CellField = CellField;
Table.Constants = Const;
Table.createCellField = createCellField;

module.exports = Table;