'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var deepcopy = require('lodash/cloneDeep');
var Const = require('uxcore-const');
var util = require('./util');

function addEmptyRow(cb) {
  this.insertRecords({}, cb);
}

function addRow(rowData, cb) {
  this.insertRecords(rowData, cb);
}

function resetRow(rowData, cb) {
  var me = this;
  var updateData = {};
  var data = me.data.datas || me.data.data;
  for (var i = 0; i < data.length; i++) {
    if (data[i].jsxid === rowData.jsxid) {
      updateData = deepcopy(data[i]);
      break;
    }
  }
  updateData.__mode__ = Const.MODE.EDIT;
  this.updateRecord(updateData, cb);
}

function delRow(rowData, cb) {
  this.removeRecords(rowData, cb);
}

function editRow(rowData, cb) {
  var newRowData = deepcopy(rowData);
  newRowData.__mode__ = Const.MODE.EDIT;
  this.updateRecord(newRowData, cb);
}

function viewRow(rowData, cb) {
  var newRowData = deepcopy(rowData);
  newRowData.__mode__ = Const.MODE.VIEW;
  this.updateRecord(newRowData, cb);
}

function saveRow(rowData, cb) {
  var newRowData = deepcopy(rowData);
  newRowData.__mode__ = Const.MODE.VIEW;
  newRowData.__edited__ = true;
  this.syncRecord(newRowData, cb);
}

function saveAllRow(cb) {
  var me = this;
  var data = deepcopy(me.state.data.data || me.state.data.datas);
  // data.forEach((item) => {
  //   me.saveRow(item);
  // });
  for (var i = 0; i < data.length; i++) {
    var item = data[i];
    item.__mode__ = Const.MODE.VIEW;
    item.__edited__ = true;
  }
  this.syncRecord(data, cb);
}

function editAllRow(cb) {
  var me = this;
  var data = deepcopy(me.data.data || me.data.datas);
  for (var i = 0; i < data.length; i++) {
    var item = data[i];
    item.__mode__ = Const.MODE.EDIT;
  }
  this.updateRecord(data, cb);
}

function toggleSubComp(rowData, cb) {
  var content = deepcopy(this.state.data);
  var data = content.data || content.datas;

  if (data) {
    for (var i = 0; i < data.length; i++) {
      var item = data[i];
      if (item.jsxid === rowData.jsxid) {
        item.showSubComp = !item.showSubComp;
        break;
      }
    }
    this.setState({
      data: content
    }, function () {
      if (cb) {
        cb();
      }
    });
  }
}

function moveRowUp(rowData, cb) {
  var content = deepcopy(this.state.data);
  var data = content.data || content.datas;
  if (data) {
    var treeIdArr = rowData.__treeId__.split('-');
    var rows = data;
    for (var i = 0; i < treeIdArr.length - 1; i++) {
      var rowIndex = treeIdArr[i];
      rows = rows[rowIndex].data;
    }
    var lastIndex = treeIdArr.pop();
    if (lastIndex > 0) {
      rows.splice(lastIndex, 1);
      // change treeId before setState;
      rows[lastIndex - 1].__treeId__ = treeIdArr.concat([lastIndex]).join('-');
      rows.splice(lastIndex - 1, 0, _extends({}, rowData, { __treeId__: treeIdArr.concat([lastIndex - 1]).join('-') }));
      this.data = content;
      this.setState({
        data: content
      }, function () {
        if (cb) {
          cb();
        }
      });
    }
  }
}

function moveRowDown(rowData, cb) {
  var content = deepcopy(this.state.data);
  var data = content.data || content.datas;
  if (data) {
    var treeIdArr = rowData.__treeId__.split('-').map(function (item) {
      return parseInt(item, 10);
    });
    var rows = data;
    for (var i = 0; i < treeIdArr.length - 1; i++) {
      var rowIndex = treeIdArr[i];
      rows = rows[rowIndex].data;
    }
    var lastIndex = treeIdArr.pop();
    if (lastIndex < rows.length - 1) {
      // change treeId before setState;
      rows[lastIndex + 1].__treeId__ = treeIdArr.concat([lastIndex]).join('-');
      rows.splice(lastIndex, 1);
      rows.splice(lastIndex + 1, 0, _extends({}, rowData, { __treeId__: treeIdArr.concat([lastIndex + 1]).join('-') }));
      this.data = content;
      this.setState({
        data: content
      }, function () {
        if (cb) {
          cb();
        }
      });
    }
  }
}

function getData(validate) {
  var me = this;
  var pass = true;
  if (validate !== false) {
    var fieldKeys = Object.keys(me.fields);
    fieldKeys.forEach(function (name) {
      var fieldPass = me.fields[name]();
      // if one field fails to pass, the table fails to pass
      if (pass) {
        pass = fieldPass;
      }
    });
  }
  if (me.props.getSavedData) {
    // 滤除可能为空的元素
    var data = deepcopy(me.data);
    data.data = data.data.filter(function (item) {
      return item !== undefined;
    });
    return {
      data: data,
      pass: pass
    };
  }
  return {
    data: me.state.data,
    pass: pass
  };
}

function toggleTreeExpanded(rowData, cb) {
  var expandedKeys = deepcopy(this.state.expandedKeys);
  util.toggleItemInArr(rowData.jsxid, expandedKeys);
  this.setState({
    expandedKeys: expandedKeys
  }, function () {
    if (cb) {
      cb();
    }
  });
}

module.exports = {
  addEmptyRow: addEmptyRow,
  addRow: addRow,
  resetRow: resetRow,
  delRow: delRow,
  editAllRow: editAllRow,
  editRow: editRow,
  viewRow: viewRow,
  saveAllRow: saveAllRow,
  saveRow: saveRow,
  toggleSubComp: toggleSubComp,
  toggleTreeExpanded: toggleTreeExpanded,
  getData: getData,
  moveRowUp: moveRowUp,
  moveRowDown: moveRowDown
};